package eu.clarin.cmdi.mdservice.proxy;

import java.io.IOException;
import java.io.InputStream;
import java.net.MalformedURLException;
import java.net.URL;
//import java.net.URLConnection;
import java.util.Map;

import org.apache.log4j.Logger;
import eu.clarin.cmdi.mdservice.action.GenericAction;
import eu.clarin.cmdi.mdservice.internal.CQLParseException;
import eu.clarin.cmdi.mdservice.internal.NoStylesheetException;
//import eu.clarin.cmdi.mdservice.internal.Utils;
import eu.clarin.cmdi.smc.SMC;

/**
 * Special proxy for SMC
 * @since 2011-10-31   
 * @author vronk
 *
 */
public class SMCProxy implements ProxyInterface{

	private static final long serialVersionUID = 1L;
	private static Logger log = Logger.getLogger("SMCProxy");
 
	private String proxy_key = "smc";
	
	private GenericAction source_action;
	
	public GenericAction getSourceAction() {
			return source_action;
	}
	
	@Override
	public void setSourceAction(GenericAction action) {		
		source_action = action;
		
	}
	
	public Map<String,String[]> getParams() {
		return getSourceAction().getParams();
	}
	
	/**
	 * Convenience function to access the request-parameters in the Action  
	 * @param key
	 * @return
	 */
	public String getParam(String key) {
		return getSourceAction().getParam(key);
	}
	
	public void checkParams() {
		// required Params	
	}
	
	/**
	 * identification of the target-proxy (type)
	 * base for finding the right base_url in the props
	 * subclass has to override with its specific proxykey
	 * @return
	 */
	public String getProxyKey() {				
			return proxy_key;		
	}

	/**
	 * Get the base_uri based on the action-key
	 * FIXME: this is provisional, what to use as base URI for smc?
	 */		
	public String getBaseURI() {		
		//String uri =Utils.getConfig).getProperty(getSourceAction().getActionkey() + ".uri");	
		//String uri = Termset.getTermsetAttr(getSourceAction().getActionkey(), "url");
		return "http://localhost:8080/MDService2/smc/";
	}

	/**
	 * Provides the base-URL specific to given target repository or registry 
	 * @return 
	 * @throws MalformedURLException
	 */	
	public URL getBaseURL() throws MalformedURLException {		
		URL base_url = new URL(getBaseURI());		
		return base_url;
	}
	
	
	/**
	 * uses base_url + url_pattern (parametrized by actionkey) to form a url
	 * @return the full target-request URL to be sent to the target repository
	 * @throws CQLParseException 
	 * @throws Exception 
	 */	
	public URL getTargetRequest() throws IOException, CQLParseException {
		 
		URL targetURL =new URL(getBaseURI() + this.getSourceAction().getTargetRequestParams());
		
		log.debug("getTargetRequest().targetURL:" + targetURL);
	    return targetURL;
		
	}

	
	public InputStream getSourceStream() throws IOException, NoStylesheetException, CQLParseException {
		SMC smc = new SMC();
	
		/* FIXME: this is inconsistent/redundant: the parameters are added via addParam(), 
		 * but context/term are also sent as parameter of the smc-functions */		
	  	smc.addParam("operation", getSourceAction().getActionkey());	 
		smc.addParam("context", getParam("context") );
		smc.addParam("term", getParam("term") );	
		smc.addParam("lang", getParam("lang") );
		
		if  (getSourceAction().getActionkey().equals("list")) {
			return smc.listTerms(getParam("context"));		
		} else if  (getSourceAction().getActionkey().equals("map")) {
			return smc.map(getParam("term"));
		} else {
			// TODO: return diagnostics!!
			return null;
		}
			
	}

	

}	